
# THIS FILE IS GENERATED FROM PYRAD_PROC SETUP.PY
short_version = '1.3.2'
version = '1.3.2'
full_version = '1.3.2'
git_revision = '7ffe3891028cffd3eeeac89617560e9c4d4bbcad'
compile_date_time = '2022-08-16 20:04'
username = 'wolfensb'
release = True

if not release:
    version = full_version
